// Copyright 1993-1995 by Jon Dart.  All Rights Reserved.

#ifndef _BOOK_ENTRY_H
#define _BOOK_ENTRY_H

#include "types.h"
#include "hashable.h"
#include "pool.h"
#include <malloc.h>

#define INVALID (uint16)0xffff

class Book_Entry : public Hashable
{
     // defines one entry in the opening book's hash table.

public:

     Book_Entry()
     {
     }

     Book_Entry( unsigned long hc, byte rec, byte mv_indx,
	         Boolean last = FALSE);

     void init( unsigned long hc, byte rec, byte mv_indx,
	        Boolean last = FALSE);
 
     // two entries are equivalent if they have the same hash code bytes.
     BOOL is_equal( const Hashable &c) const;
	    
     void *operator new (size_t size)
     {
	return allocator.allocate(size);
     }
    
     void *operator new (size_t, void *p)
     {
	return p;    
     }

     void operator delete( void *p )
     {
	allocator.free(p);    
     }
    
     static void freeAll( BOOL final )
     {
	allocator.freeAll(final);
     }

     unsigned long hash_code() const
     {
        return my_hash_code;
     }
     
     byte get_move_index() const
     { 
	return move_index;
     }
     
     Boolean is_last() const;

     void set_last(Boolean b = TRUE);

     unsigned get_recommend() const;

     unsigned long my_hash_code;
     byte move_index;
private:
     byte recommend;
     static Pool allocator;
};

#endif
