// Copyright 1993 by Jon Dart.  All Rights Reserved.

#include "stdafx.h"
#include "bookread.h"
#include "constant.h"
#include "bhash.h"
#include <string.h>

HINSTANCE Book_Reader::hLibInstance = 0;
UINT Book_Reader::refCount = 0;

const int Entry_Size = 6; //sizeof(Book_Entry);
const int Header_Size = 8;
// version of opening book, change it if the format changes:
const int Book_Version = 3;
#ifdef _WIN32
static const char bookdll[] = "book32.dll";
#else 
static const char bookdll[] = "book16.dll";
#endif

Book_Reader::Book_Reader(const char *const filename)
{                 
   pBook = NULL;

   if (!hLibInstance)
   {
      hLibInstance = LoadLibrary(bookdll);  
      if (hLibInstance)
         refCount++;
   }
   TRACE("LoadLibrary: result = %d\n",hLibInstance);
//   hBook = FindResource(AfxGetResourceHandle(),filename,"BOOK");
   hBook = FindResource(hLibInstance,filename,"BOOK");
   TRACE("FindResource: result = %d\n",hBook);
   if (hBook)
      pBook = LoadResource(hLibInstance,hBook);
   is_open = pBook != NULL;
   if (!is_open)
      return;
   char version;
#ifdef _WIN32
   LPSTR lpBook = (LPSTR)pBook;
#else
   LPSTR lpBook = (LPSTR)LockResource(pBook);
#endif
   ASSERT(lpBook);
   version = lpBook[0];
   if (version != Book_Version)
   {
      MessageBox(NULL,"Wrong version of opening book!","",MB_OK);
#ifndef _WIN32
      UnlockResource(pBook);
#endif
      is_open = False;
      return;
   }
   unsigned char lo_size = lpBook[1];
   unsigned char hi_size = lpBook[2];
   my_hash_size = hi_size*256 + lo_size;
   lo_size = lpBook[3];
   hi_size = lpBook[4];
   my_size = hi_size*256 + lo_size;
#ifndef _WIN32
   UnlockResource(pBook);
#endif
}

Book_Reader::~Book_Reader()
{
#ifndef _WIN32
   FreeResource(pBook);
#endif
   if (--refCount == 0)
   {
      FreeLibrary(hLibInstance);
      hLibInstance = 0;
   }
}

const uint16 Book_Reader::Head( const Board & b)
{
#ifdef _WIN32
   LPCSTR lpBook = (LPCSTR)pBook;
#else                                           
   LPCSTR lpBook = (LPCSTR)LockResource(pBook);
#endif                                                
   ASSERT(lpBook);
   
   unsigned probe = (unsigned)(Board_Hash::HashCode2(b) % my_hash_size);
   uint16 ret_val;
   memcpy(&ret_val,lpBook+Header_Size+sizeof(uint16)*probe,sizeof(uint16));
#ifndef _WIN32
   UnlockResource(pBook);
#endif
   return ret_val;
}

void Book_Reader::Fetch( const uint16 n, Book_Entry &book_entry )
{
//#ifdef RANGE_CHECK
   ASSERT(n<my_size);
//#endif
#ifdef _WIN32
   byte *lpBook = (byte*)pBook;
#else
   byte *lpBook = (byte*)LockResource(pBook);
#endif
   ASSERT(lpBook);
   byte *entry = lpBook + Header_Size + my_hash_size*2 + n*Entry_Size;
   unsigned long hc;
   byte rec, indx;
   memcpy(&hc,entry,sizeof(unsigned long));
   indx = entry[4];
   rec = entry[5];
   Boolean last = (rec & 0x80) ? True : False;
   book_entry.init(hc,rec & 0xf,indx, last);
#ifndef _WIN32
   UnlockResource(pBook);
#endif
}


